package burp.vaycore.common.helper;

import burp.vaycore.common.utils.FileUtils;
import burp.vaycore.common.utils.GsonUtils;
import burp.vaycore.common.utils.IPUtils;
import burp.vaycore.common.utils.StringUtils;
import java.io.InputStream;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class DomainHelper {

   private static Map sTree;


   private DomainHelper() {
      throw new IllegalAccessError("DomainHelper class not support create instance.");
   }

   public static void init(String resName) {
      InputStream is = DomainHelper.class.getClassLoader().getResourceAsStream(resName);
      String json = FileUtils.readStreamToString(is);
      sTree = GsonUtils.toMap(json);
   }

   public static String getDomain(String fqdn) {
      return getDomain(fqdn, fqdn);
   }

   public static String getDomain(String fqdn, String defValue) {
      if(IPUtils.hasIPv4(fqdn)) {
         return defValue;
      } else {
         String[] split = fqdn.split("\\.");
         List list = Arrays.asList(split);
         LinkedList parts = new LinkedList(list);
         String domain = queryDomain(parts, sTree);
         return !StringUtils.isEmpty(domain) && domain.contains(".")?domain:defValue;
      }
   }

   public static String getDomainName(String fqdn) {
      return getDomainName(fqdn, fqdn);
   }

   public static String getDomainName(String fqdn, String defValue) {
      String domain = getDomain(fqdn, (String)null);
      return domain == null?defValue:domain.split("\\.")[0];
   }

   private static String queryDomain(LinkedList parts, Map node) {
      if(parts.size() == 0) {
         return null;
      } else {
         String sub = (String)parts.removeLast();
         if(node.get("!") != null) {
            return "";
         } else {
            String result;
            if(node.get(sub) != null) {
               result = queryDomain(parts, (Map)node.get(sub));
            } else {
               if(node.get("*") == null) {
                  return sub;
               }

               result = queryDomain(parts, (Map)node.get("*"));
            }

            return result == null?null:(result.equals("")?sub:result + "." + sub);
         }
      }
   }
}
